@@ -3,7 +3,7 @@
 # =========================================================================
 #   Program:   S1Processor
 #
-#   Copyright 2017-2021 (c) CESBIO. All rights reserved.
+#   Copyright 2017-2021 (c) CNES. All rights reserved.
 #
 #   This file is part of S1Tiling project
 #       https://gitlab.orfeo-toolbox.org/s1-tiling/s1tiling
@@ -31,15 +31,16 @@
 """ This module contains the S1FileManager class"""
 
 import fnmatch
+from functools import partial
 import glob
 import logging
+import multiprocessing
 import os
 from pathlib import Path
 import re
 import shutil
 import sys
 import tempfile
-import zipfile
 
 from eodag.api.core import EODataAccessGateway
 from eodag.utils.logging import setup_logging
@@ -49,6 +50,7 @@ import numpy as np
 from s1tiling.libs import exits
 from .Utils import get_shape, list_files, list_dirs
 from .S1DateAcquisition import S1DateAcquisition
+from .otbpipeline import mp_worker_config
 
 setup_logging(verbose=1)
 
@@ -181,6 +183,55 @@ def discard_small_redundant(products, id=None):
     return res
 
 
+def _download_and_extract_one_product(dag, raw_directory, product):
+    """
+    Takes care of downloading exactly one remote product and unzipping it,
+    if required.
+
+    Some products are already unzipped on the fly by eodag.
+    """
+    logging.info("Starting download of %s...", product)
+    ok_msg = "Successful download (and extraction) of %s" % (product, )  # because eodag'll clear product
+    file = os.path.join(raw_directory, product.as_dict()['id']) + '.zip'
+    path = dag.download(
+            product,      # EODAG will clear this variable
+            extract=True  # Let's eodag do the job
+            )
+    logging.debug(ok_msg)
+    if os.path.exists(file) :
+        try:
+            logger.debug('Removing downloaded ZIP: %s', file)
+            os.remove(file)
+        except OSError:
+            pass
+    return path
+
+
+def _parallel_download_and_extraction_of_products(dag, raw_directory, products, nb_procs, tile_name):
+    """
+    Takes care of downloading exactly all remote products and unzipping them,
+    if required, in parallel.
+    """
+    paths = []
+    log_queue = multiprocessing.Queue()
+    log_queue_listener = logging.handlers.QueueListener(log_queue)
+    dl_work = partial(_download_and_extract_one_product, dag, raw_directory)
+    with multiprocessing.Pool(nb_procs, mp_worker_config, [log_queue]) as pool:
+        log_queue_listener.start()
+        try:
+            for count, result in enumerate(pool.imap_unordered(dl_work, products), 1):
+                logger.info("%s correctly downloaded", result)
+                logger.info(' --> Downloading products for %s... %s%%', tile_name, count * 100. / len(products))
+                paths.append(result)
+        finally:
+            pool.close()
+            pool.join()
+            log_queue_listener.stop()  # no context manager for QueueListener unfortunately
+
+    # paths returns the list of .SAFE directories
+    return paths
+
+
 class S1FileManager:
     """ Class to manage processed files (downloads, checks) """
     def __init__(self, cfg):
@@ -189,6 +240,8 @@ class S1FileManager:
         self.nb_images        = 0
 
         self.__tmpsrtmdir     = None
+        self.__caching_option = cfg.cache_srtm_by
+        assert self.__caching_option in ['copy', 'symlink']
 
         self.tiff_pattern     = "measurement/*.tiff"
         self.vh_pattern       = "measurement/*vh*-???.tiff"
@@ -244,7 +297,7 @@ class S1FileManager:
         all exist
         """
         for path in [self.cfg.raw_directory, self.cfg.tmpdir, self.cfg.output_preprocess]:
-            if not os.path.exists(path):
+            if not os.path.isdir(path):
                 os.makedirs(path, exist_ok=True)
 
     def ensure_tile_workspaces_exist(self, tile_name):
@@ -263,9 +316,11 @@ class S1FileManager:
 
     def tmpsrtmdir(self, srtm_tiles_id, srtm_suffix='.hgt'):
         """
-        Generate the temporary directory for SRTM tiles on the fly
-        And populate it with symbolic link to the actual SRTM tiles
+        Generate the temporary directory for SRTM tiles on the fly,
+        and either populate it with symbolic links to the actual SRTM
+        tiles, or copies of the actual SRTM tiles.
         """
+        assert self.__caching_option in ['copy', 'symlink']
         if not self.__tmpsrtmdir:
             # copy all needed SRTM file in a temp directory for orthorectification processing
             self.__tmpsrtmdir = tempfile.TemporaryDirectory(dir=self.cfg.tmpdir)
@@ -274,10 +329,13 @@ class S1FileManager:
             for srtm_tile in srtm_tiles_id:
                 srtm_tile_filepath=Path(self.cfg.srtm, srtm_tile + srtm_suffix)
                 srtm_tile_filelink=Path(self.__tmpsrtmdir.name, srtm_tile + srtm_suffix)
-                logger.debug('ln -s %s  <-- %s',
-                    srtm_tile_filepath,
-                    srtm_tile_filelink)
-                srtm_tile_filelink.symlink_to(srtm_tile_filepath)
+                if self.__caching_option == 'symlink':
+                    logger.debug('- ln -s %s <-- %s', srtm_tile_filepath, srtm_tile_filelink)
+                    srtm_tile_filelink.symlink_to(srtm_tile_filepath)
+                else:
+                    logger.debug('- cp %s <-- %s', srtm_tile_filepath, srtm_tile_filelink)
+                    shutil.copy2(srtm_tile_filepath, srtm_tile_filelink)
+
         return self.__tmpsrtmdir.name
 
     def keep_X_latest_S1_files(self, threshold):
@@ -298,7 +356,7 @@ class S1FileManager:
             lonmin, lonmax, latmin, latmax,
             first_date, last_date,
             tile_out_dir, tile_name, polarization,
-            searched_items_per_page, dryrun):
+            searched_items_per_page,dryrun):
         """
         Process with the call to eodag download.
         """
@@ -312,13 +370,17 @@ class S1FileManager:
         products = []
         page = 1
         while True:
+            assert polarization in ['VV VH', 'VV', 'VH', 'HH HV', 'HH', 'HV']
+            # In case only 'VV' or 'VH' is requested, we still need to
+            # request 'VV VH' to the data provider through eodag.
+            dag_polarization_param = 'VV VH' if polarization in ['VV VH', 'VV', 'VH'] else 'HH HV'
             page_products, _ = dag.search(
                     page=page, items_per_page=searched_items_per_page,
                     productType=product_type,
                     start=first_date, end=last_date,
                     box=extent,
                     # If we have eodag v1.6, we try to filter product during the search request
-                    polarizationMode=polarization,
+                    polarizationMode=dag_polarization_param,
                     sensorMode="IW"
                     )
             logger.info("%s remote S1 products returned in page %s: %s", len(page_products), page, page_products)
@@ -334,7 +396,7 @@ class S1FileManager:
         # -> This filter is required with eodag < v1.6, it's redundant w/ v1.6+
         products = [p for p in products
                 if (    product_property(p, "sensorMode",       "") == "IW"
-                    and product_property(p, "polarizationMode", "") == polarization)
+                    and product_property(p, "polarizationMode", "") == dag_polarization_param)
                 ]
         logger.debug("%s remote S1 product(s) found and filtered (IW && %s): %s", len(products), polarization, products)
         if not products:  # no need to continue
@@ -379,19 +441,11 @@ class S1FileManager:
             paths = [p.as_dict()['id'] for p in products] # TODO: return real name
             logger.info("Remote S1 products would have been saved into %s", paths)
             return paths
-        paths = dag.download_all(
-                products[:],  # pass a copy because eodag modifies the list
-                )
-        # paths returns the list of .SAFE directories
+
+        paths = _parallel_download_and_extraction_of_products(
+                dag, self.cfg.raw_directory, products, self.cfg.nb_download_processes,
+                tile_name)
         logger.info("Remote S1 products saved into %s", paths)
-        # And clean temporary files
-        for product in products:
-            file = os.path.join(self.cfg.raw_directory, product.as_dict()['id']) + '.zip'
-            try:
-                logger.debug('Removing downloaded ZIP: %s', file)
-                os.remove(file)
-            except OSError:
-                pass
         return paths
 
     def download_images(self, searched_items_per_page, dryrun=False, tiles=None):
@@ -458,10 +512,10 @@ class S1FileManager:
             logger.debug("# Safe dir: %s", safe_dir)
             logger.debug("  all tiffs: %s", list(all_tiffs))
 
-            vv_images = filter_images_or_ortho('vv', all_tiffs)
-            vh_images = filter_images_or_ortho('vh', all_tiffs)
-            hv_images = filter_images_or_ortho('hv', all_tiffs)
-            hh_images = filter_images_or_ortho('hh', all_tiffs)
+            vv_images = filter_images_or_ortho('vv', all_tiffs) if self.cfg.polarisation in ['VV', 'VV VH'] else []
+            vh_images = filter_images_or_ortho('vh', all_tiffs) if self.cfg.polarisation in ['VH', 'VV VH'] else []
+            hv_images = filter_images_or_ortho('hv', all_tiffs) if self.cfg.polarisation in ['HV', 'HH HV'] else []
+            hh_images = filter_images_or_ortho('hh', all_tiffs) if self.cfg.polarisation in ['HH', 'HH HV'] else []
 
             for image in vv_images + vh_images + hv_images + hh_images:
                 if image not in self.processed_filenames:
